@@ -7,7 +7,7 @@ import re
 import sys
 import types
 import warnings
-from collections.abc import Hashable, MutableSequence, MutableMapping  # type: ignore
+from collections.abc import Hashable, MutableSequence, MutableMapping
 
 # fmt: off
 from ruamel.yaml.error import (MarkedYAMLError, MarkedYAMLFutureWarning,
@@ -16,7 +16,7 @@ from ruamel.yaml.nodes import *                               # NOQA
 from ruamel.yaml.nodes import (SequenceNode, MappingNode, ScalarNode)
 from ruamel.yaml.compat import (_F, builtins_module, # NOQA
                                 nprint, nprintf, version_tnf)
-from ruamel.yaml.compat import ordereddict  # type: ignore
+from ruamel.yaml.compat import ordereddict
 
 from ruamel.yaml.comments import *                               # NOQA
 from ruamel.yaml.comments import (CommentedMap, CommentedOrderedMap, CommentedSet,
@@ -51,7 +51,7 @@ class DuplicateKeyFutureWarning(MarkedYAMLFutureWarning):
     pass
 
 
-class DuplicateKeyError(MarkedYAMLFutureWarning):
+class DuplicateKeyError(MarkedYAMLError):
     pass
 
 
@@ -545,10 +545,7 @@ class SafeConstructor(BaseConstructor):
                 node.start_mark,
             )
         try:
-            if hasattr(base64, 'decodebytes'):
-                return base64.decodebytes(value)
-            else:
-                return base64.decodestring(value)
+            return base64.decodebytes(value)
         except binascii.Error as exc:
             raise ConstructorError(
                 None,
@@ -804,10 +801,7 @@ class Constructor(SafeConstructor):
                 node.start_mark,
             )
         try:
-            if hasattr(base64, 'decodebytes'):
-                return base64.decodebytes(value)
-            else:
-                return base64.decodestring(value)
+            return base64.decodebytes(value)
         except binascii.Error as exc:
             raise ConstructorError(
                 None,
@@ -1068,12 +1062,14 @@ class RoundTripConstructor(SafeConstructor):
     """
 
     def comment(self, idx):
+        # type: (Any) -> Any
         assert self.loader.comment_handling is not None
         x = self.scanner.comments[idx]
         x.set_assigned()
         return x
 
     def comments(self, list_of_comments, idx=None):
+        # type: (Any, Optional[Any]) -> Any
         # hand in the comment and optional pre, eol, post segment
         if list_of_comments is None:
             return []
@@ -1103,7 +1099,8 @@ class RoundTripConstructor(SafeConstructor):
                 # NEWCMNT
                 if node.comment is not None and node.comment[1]:
                     # nprintf('>>>>nc1', node.comment)
-                    lss.comment = self.comment(node.comment[1][0])  # EOL comment after |
+                    # EOL comment after |
+                    lss.comment = self.comment(node.comment[1][0])  # type: ignore
             return lss
         if node.style == '>' and isinstance(node.value, str):
             fold_positions = []  # type: List[int]
@@ -1121,7 +1118,8 @@ class RoundTripConstructor(SafeConstructor):
                 # NEWCMNT
                 if node.comment is not None and node.comment[1]:
                     # nprintf('>>>>nc2', node.comment)
-                    lss.comment = self.comment(node.comment[1][0])  # EOL comment after >
+                    # EOL comment after >
+                    lss.comment = self.comment(node.comment[1][0])  # type: ignore
             if fold_positions:
                 fss.fold_pos = fold_positions  # type: ignore
             return fss
@@ -1727,7 +1725,10 @@ class RoundTripConstructor(SafeConstructor):
                 data.yaml_set_tag(node.tag)
                 yield data
                 if node.anchor:
-                    data.yaml_set_anchor(node.anchor)
+                    from ruamel.yaml.serializer import templated_id
+
+                    if not templated_id(node.anchor):
+                        data.yaml_set_anchor(node.anchor)
                 self.construct_mapping(node, data)
                 return
             elif isinstance(node, ScalarNode):
@@ -1737,7 +1738,10 @@ class RoundTripConstructor(SafeConstructor):
                 data2.yaml_set_tag(node.tag)
                 yield data2
                 if node.anchor:
-                    data2.yaml_set_anchor(node.anchor, always_dump=True)
+                    from ruamel.yaml.serializer import templated_id
+
+                    if not templated_id(node.anchor):
+                        data2.yaml_set_anchor(node.anchor, always_dump=True)
                 return
             elif isinstance(node, SequenceNode):
                 data3 = CommentedSeq()
@@ -1749,7 +1753,10 @@ class RoundTripConstructor(SafeConstructor):
                 data3.yaml_set_tag(node.tag)
                 yield data3
                 if node.anchor:
-                    data3.yaml_set_anchor(node.anchor)
+                    from ruamel.yaml.serializer import templated_id
+
+                    if not templated_id(node.anchor):
+                        data3.yaml_set_anchor(node.anchor)
                 data3.extend(self.construct_sequence(node))
                 return
         except:  # NOQA
